package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microservice.goods.facade.clients.GoodsBatchGradeManagerService;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchGradeForm;
import com.hcloud.microservice.goods.service.GoodsBatchGradeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/goods/grade")
public class GoodsBatchGradeManageController extends BaseController implements GoodsBatchGradeManagerService {


    @Autowired
    private GoodsBatchGradeService goodsBatchGradeService;

    /**
     * 分页查询商品评级信息
     *
     * @param gradeForm
     * @return
     */
    @PostMapping("/searchBatchGradeByPage")
    @Override
    public ResultVO searchBatchGradeByPage(@RequestBody GoodsBatchGradeForm gradeForm) {
        PageInfo pageInfo = goodsBatchGradeService.searchBatchGradeByPage(gradeForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 获取用户评级信息
     *
     * @param userId
     * @return
     */
    @GetMapping("/getBatchGradeByUserId")
    @Override
    public ResultVO getBatchGradeByUserId(@RequestParam("userId") String userId,
                                          @RequestParam("batchId") String batchId) {
        List<GoodsBatchGradeForm> list = goodsBatchGradeService.getBatchGradeInfoByCond(userId,batchId);
        return success(list);
    }

    /**
     * 保存用户评级信息
     *
     * @param gradeForm
     * @return
     */
    @PostMapping("/saveBatchGradeInfo")
    @Override
    public ResultVO saveBatchGradeInfo(@RequestBody GoodsBatchGradeForm gradeForm) {
       int i = goodsBatchGradeService.saveSelective(gradeForm);
       if(i>0){
    	   return success();
       }
        return failure();
    }

    /**
     * 批量删除商品评级信息
     *
     * @param gradeForm
     * @return
     */
    @PostMapping("/delBatchGradeInfo")
    @Override
    public ResultVO delBatchGradeInfo(@RequestBody GoodsBatchGradeForm gradeForm) {
        if(gradeForm.getPkList()!=null && gradeForm.getPkList().size()>0){
            int i = goodsBatchGradeService.delBatchGradeInfo(gradeForm.getPkList());
            if(i>0){
                return success();
            }
        }
        return failure();
    }
}
