package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microservice.goods.facade.clients.GoodsFeedbackManagerService;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchFeedbackForm;
import com.hcloud.microservice.goods.service.GoodsBatchFeedbackService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author JYJ
 */
@RestController
@RequestMapping("/goods/feedback")
public class GoodsFeedbackManageController extends BaseController implements GoodsFeedbackManagerService {

    @Autowired
    private GoodsBatchFeedbackService goodsBatchFeedbackService;

    /**
     * 分页查询商品评价信息
     *
     * @param gradeForm
     * @return
     */
    @PostMapping("/searchGoodsFeedbackByPage")
    @Override
    public ResultVO searchGoodsFeedbackByPage(@RequestBody GoodsBatchFeedbackForm gradeForm) {
        PageInfo pageInfo =  goodsBatchFeedbackService.searchBatchFeedbackByPage(gradeForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 获取用户评价信息
     * @param userId
     * @return
     */
    @GetMapping("/getGoodsFeedbackByUserId")
    @Override
    public ResultVO getGoodsFeedbackByUserId(@RequestParam("userId") String userId,@RequestParam("batchId") String batchId) {
        List<GoodsBatchFeedbackForm> resultList = goodsBatchFeedbackService.getGoodsFeedbackByUserId(userId,batchId);
        return success(resultList);
    }

    /**
     * 保存用户评价信息
     * @param gradeForm
     * @return
     */
    @PostMapping("/saveGoodsFeedbackInfo")
    @Override
    public ResultVO saveGoodsFeedbackInfo(@RequestBody GoodsBatchFeedbackForm gradeForm) {
        int i = goodsBatchFeedbackService.saveSelective(gradeForm);
        if(i>0){
            return success();
        }
        return failure();
    }

    /**
     * 批量删除用户评价
     * @param feedbackForm
     * @return
     */
    @PostMapping("/delGoodsFeedbackInfo")
    @Override
    public ResultVO delGoodsFeedbackInfo(@RequestBody GoodsBatchFeedbackForm feedbackForm) {
        if(feedbackForm.getPkList()!=null && feedbackForm.getPkList().size()>0){
           int i =  goodsBatchFeedbackService.delGoodsFeedbackByList(feedbackForm.getPkList());
            if(i>0){
                return success();
            }
        }
        return failure();
    }
}
