package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microservice.goods.facade.clients.GoodsQRCodeRecordManagerService;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeInfoForm;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordForm;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordStatForm;
import com.hcloud.microservice.goods.facade.results.ScanNumStatResult;
import com.hcloud.microservice.goods.facade.results.ScanNumStatisResult;
import com.hcloud.microservice.goods.service.GoodsQRCodeRecordService;
import com.hcloud.microservice.goods.service.GoodsQRCodeRecordStatService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedList;
import java.util.List;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/qrcode/record")
public class GoodsQRCodeRecordController extends BaseController implements GoodsQRCodeRecordManagerService {

    @Autowired
    private GoodsQRCodeRecordService goodsQRCodeRecordService;

    @Autowired
    private GoodsQRCodeRecordStatService goodsQRCodeRecordStatService;

    /**
     * 分页查询溯源码生成记录
     *
     * @param goodsQRCodeRecordForm
     * @return
     */
    @PostMapping("/searchByPage")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsQRCodeRecordByPage(@RequestBody GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        PageInfo pageInfo = goodsQRCodeRecordService.searchQRCodeRecordByPage(goodsQRCodeRecordForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 查询溯源码生成记录为导出
     *
     * @param goodsQRCodeRecordForm
     * @return
     */
    @PostMapping("/searchForExport")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsQRCodeRecordForExport(@RequestBody GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        GoodsQRCodeInfoForm queryInfo = new GoodsQRCodeInfoForm();
        queryInfo.setPageSize(goodsQRCodeRecordForm.getPageSize());
        queryInfo.setFkBatchRecordGuid(goodsQRCodeRecordForm.getGuid());
        PageInfo<GoodsQRCodeRecordForm> pageInfo = goodsQRCodeRecordService.getGoodsQRCodeDetailById(queryInfo);
        return success(pageInfo.getList());
    }

    /**
     * 批量删除溯源码生成记录
     * @param goodsQRCodeRecordForm
     * @return
     */
    @PostMapping("/delGoodsQRCodeRecord")
    @AuthOrgValidate
    @Override
    public ResultVO delGoodsQRCodeRecord(@RequestBody GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        int i = goodsQRCodeRecordService.delGoodsQRCodeRecord(goodsQRCodeRecordForm);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 通过recordId查询生成记录明细
     *
     * @param goodsQRCodeRecordForm
     * @return
     */
    @PostMapping("/getGoodsQRCodeInfoByRecordId")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsQRCodeInfoByRecordId(@RequestBody GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        GoodsQRCodeInfoForm queryInfo = new GoodsQRCodeInfoForm();
        queryInfo.setFkBatchRecordGuid(goodsQRCodeRecordForm.getGuid());
        queryInfo.setPageNum(goodsQRCodeRecordForm.getPageNum());
        queryInfo.setPageSize(goodsQRCodeRecordForm.getPageSize());
        PageInfo pageInfo = goodsQRCodeRecordService.getGoodsQRCodeDetailById(queryInfo);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 物理删除溯源码生成记录
     *
     * @param recordId
     * @return
     */
    @Override
    @GetMapping("/delQRCodeInfoPhysical")
    @AuthOrgValidate
    public ResultVO delQRCodeInfoPhysical(@RequestParam("recordId") String recordId) {
        int i = goodsQRCodeRecordService.delQRcodeInfoByPhysical(recordId);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 统计全年各月的扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    @Override
    @AuthOrgValidate
    @PostMapping("/statOneYear")
    public ResultVO statOneYearScanNumber(@RequestBody GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
    	LinkedList<ScanNumStatisResult> result = this.goodsQRCodeRecordStatService.statOneYearAllMonthData(goodsQRCodeRecordStatForm);
        return success(result);
    }

    /**
     * 统计昨日和历史累计扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    @Override
    @AuthOrgValidate
    @PostMapping("/statYesterdayAndAllTime")
    public ResultVO statYesterdayAndAllTimeScanNumber(@RequestBody GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordStatService.statYesterdayAndAllTimeData(goodsQRCodeRecordStatForm);
        return success(results);
    }

    @Override
    @AuthOrgValidate
    @PostMapping("/statOneDayAllHours")
    public ResultVO statOneDayAllHoursScanNumber(@RequestBody GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordStatService.statOneDayAllHoursData(goodsQRCodeRecordStatForm);
        return success(results);
    }

    @Override
    @AuthOrgValidate
    @PostMapping("/statOneWeekSevenDay")
    public ResultVO statOneWeekSevenDayScanNumber(@RequestBody GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordStatService.statOneWeekSevenDaysData(goodsQRCodeRecordStatForm);
        return success(results);
    }

    @Override
    @AuthOrgValidate
    @PostMapping("/statOneMonthAllDays")
    public ResultVO statOneMonthAllDaysScanNumber(@RequestBody GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordStatService.statOneMonthAllDaysData(goodsQRCodeRecordStatForm);
        return success(results);
    }
}
