package com.hcloud.microservice.goods.service.impl;

import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.commoncore.util.string.ConvertStringToList;
import com.hcloud.microservice.goods.dao.GoodsAssayInfoMapper;
import com.hcloud.microservice.goods.dao.GoodsInfoAuxiliaryMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsAssayInfo;
import com.hcloud.microservice.goods.facade.entity.GoodsInfoAuxiliary;
import com.hcloud.microservice.goods.facade.forms.GoodsAssayInfoForm;
import com.hcloud.microservice.goods.facade.forms.GoodsInfoAuxiliaryForm;
import com.hcloud.microservice.goods.service.GoodsAssayInfoService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author JYJ
 */
@Transactional(rollbackFor = Exception.class)
@Service("goodsAssayInfService")
public class GoodsAssayInfServiceImpl implements GoodsAssayInfoService {

    @Resource
    private GoodsAssayInfoMapper goodsAssayInfoMapper;

    @Resource
    private GoodsInfoAuxiliaryMapper goodsInfoAuxiliaryMapper;

    /**
     * 保存商品批次检疫检验信息
     *
     * @param goodsAssayInfoForm
     * @return
     */
    @Override
    public int saveBatchGoodsAssayInfo(GoodsAssayInfoForm goodsAssayInfoForm) {
        //TODO 保存检验主表
        if(goodsAssayInfoForm!=null){
            GoodsAssayInfo goodsAssayInfo = new GoodsAssayInfo();
            BeanCopyUtil.copyBean(goodsAssayInfoForm,goodsAssayInfo);
            int i = this.saveSelective(goodsAssayInfo);
                List<GoodsInfoAuxiliaryForm> auxList = goodsAssayInfoForm.getAuxList();
                //TODO 判断是否有自定义属性字段
                if(null!=auxList && auxList.size()>0){
                    for (GoodsInfoAuxiliaryForm aux:auxList) {
                        aux.setFkOrgGuid(goodsAssayInfoForm.getFkOrgGuid());
                        aux.setFkAuxItmeId(goodsAssayInfo.getGuid());
                        aux.setFkGoodsGuid(goodsAssayInfoForm.getFkGoodsBatchId());
                        aux.setState(goodsAssayInfoForm.getState());
                        aux.setCreateUser(goodsAssayInfoForm.getCreateUser());
                        aux.setCreateTime(new Date());
                        aux.setModifiedUser(goodsAssayInfoForm.getModifiedUser());
                        aux.setModifiedTime(new Date());
                    }
                    goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
                }
            return i;
            }
        return 0;
    }

    /**
     * 通过商品批次id查询商品批次检疫检验信息
     *
     * @param batchId
     * @return
     */
    @Override
    public GoodsAssayInfoForm searchByGoodsBatchId(String batchId) {
        GoodsAssayInfoForm goodsAssayInfoForm = new GoodsAssayInfoForm();
        GoodsAssayInfo assayInfo =  goodsAssayInfoMapper.getGoodsAssayInfoByBatchId(batchId);
        if(assayInfo!=null){
           BeanCopyUtil.copyBean(assayInfo,goodsAssayInfoForm);
           String assayInfoId =  assayInfo.getGuid();
           String imgStr = assayInfo.getAssayImgs();
           List<GoodsInfoAuxiliary> auxList = goodsInfoAuxiliaryMapper.queryListByParentId(assayInfoId);
           List<GoodsInfoAuxiliaryForm> auxFormList =  BeanCopyUtil.convertList(auxList,GoodsInfoAuxiliaryForm.class);
           //TODO 查询辅助信息
            goodsAssayInfoForm.setAuxList(auxFormList);
            //TODO 查询图片信息
            List<String> list = ConvertStringToList.getListFormSplitString(imgStr,",");
            goodsAssayInfoForm.setImagesList(list);
        }
        return goodsAssayInfoForm;
    }

    /**
     * 编辑商品批次检验信息
     *
     * @param goodsAssayInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int editBatchGoodsAssayInfo(GoodsAssayInfoForm goodsAssayInfoForm) {
        if(goodsAssayInfoForm!=null){
            //更新商品检验信息
            String guid = goodsAssayInfoForm.getGuid();
            String orgGuid = goodsAssayInfoForm.getFkOrgGuid();
            GoodsAssayInfo assayInfo = goodsAssayInfoMapper.selectByPrimaryKey(guid);
            if(assayInfo!=null){
                assayInfo.setState(0);
                assayInfo.setModifiedUser(goodsAssayInfoForm.getModifiedUser());
                assayInfo.setModifiedTime(new Date());
                BeanCopyUtil.copyBean(goodsAssayInfoForm,assayInfo);
               int i = goodsAssayInfoMapper.updateByPrimaryKeySelective(assayInfo);
               if(i==0){throw new RuntimeException();}
                //更新商品检验辅助信息
                List<GoodsInfoAuxiliaryForm> auxList = goodsAssayInfoForm.getAuxList();
                if(null!=auxList && auxList.size()>0){
                    for (GoodsInfoAuxiliaryForm bean:auxList) {
                        bean.setFkOrgGuid(goodsAssayInfoForm.getFkOrgGuid());
                        bean.setFkAuxItmeId(assayInfo.getGuid());
                        bean.setFkGoodsGuid(assayInfo.getGuid());
                        bean.setState(1);
                        bean.setCreateUser(goodsAssayInfoForm.getCreateUser());
                        bean.setCreateTime(goodsAssayInfoForm.getCreateTime());
                        bean.setModifiedUser(goodsAssayInfoForm.getModifiedUser());
                        bean.setModifiedTime(goodsAssayInfoForm.getModifiedTime());
                    }
                    goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(guid);
                    goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
                }
                return i;
            }
        }
        return 0;
    }

    /**
     * 删除商品批次检验信息
     *
     * @param guid
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int delBatchGoodsAssayInfo(String guid,String orgGuid) {
        GoodsAssayInfo assayInfo = goodsAssayInfoMapper.selectByPrimaryKey(guid);
        if(assayInfo!=null){
            assayInfo.setState(0);
            assayInfo.setModifiedTime(new Date());
            int i = goodsAssayInfoMapper.updateByPrimaryKeySelective(assayInfo);
            if(i==0){throw new RuntimeException();}
            goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(guid);
            return i;
        }
        return 0;
    }

    /**
     * 删除商品批次检验信息通过商品批次ID
     * @param batchId
     * @return
     */
    @Override
    public int delBatchGoodsAssayInfoByBatchId(String batchId) {
        GoodsAssayInfo assayInfo = goodsAssayInfoMapper.getGoodsAssayInfoByBatchId(batchId);
        if(null!=assayInfo){
            String assayId = assayInfo.getGuid();
            String orgGuid = assayInfo.getFkOrgGuid();
            assayInfo.setState(0);
            assayInfo.setModifiedTime(new Date());
            int i = goodsAssayInfoMapper.updateByPrimaryKeySelective(assayInfo);
            if(i==0){throw new RuntimeException();}
            goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(assayId);
        }



        return 0;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsAssayInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsAssayInfo record) {
        record.setGuid(UUIDUtils.randomUUID());
        return goodsAssayInfoMapper.insert(record);
    }

    @Override
    public int saveSelective(GoodsAssayInfo record) {
        record.setGuid(UUIDUtils.randomUUID());
        return goodsAssayInfoMapper.insertSelective(record);
    }

    @Override
    public GoodsAssayInfo queryByPrimaryKey(String guid) {
        return goodsAssayInfoMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsAssayInfo record) {
        return goodsAssayInfoMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsAssayInfo record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsAssayInfo record) {
        return goodsAssayInfoMapper.updateByPrimaryKey(record);
    }
}
