package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.exception.GlobalException;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.string.ConvertStringToList;
import com.hcloud.microservice.goods.dao.GoodsFieldMapper;
import com.hcloud.microservice.goods.dao.GoodsInfoAuxiliaryMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsField;
import com.hcloud.microservice.goods.facade.entity.GoodsInfoAuxiliary;
import com.hcloud.microservice.goods.facade.forms.GoodsFieldForm;
import com.hcloud.microservice.goods.facade.forms.GoodsInfoAuxiliaryForm;
import com.hcloud.microservice.goods.service.GoodsFieldService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author xiezt
 */
@Service("goodsFieldService")
public class GoodsFieldServiceImpl implements GoodsFieldService {

    private static final Integer POSITION_HEAD_LENGTH=6;

    private static final Integer POSITION_LEVEL_STEP=3;

    @Resource
    private GoodsFieldMapper goodsFieldMapper;

    @Resource
    private GoodsInfoAuxiliaryMapper goodsInfoAuxiliaryMapper;


    @Override
    public int delByPrimaryKey(String guid) {
        int i = goodsFieldMapper.deleteByPrimaryKey(guid);
        if(i > 0) {
        	goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(guid);
        }
        return i;
    }

    @Override
    public int save(GoodsField record) {
        record.setGuid(GetUUID.getUUID());
        return goodsFieldMapper.insert(record);
    }

    @Override
    public int saveSelective(GoodsField record) {
        return goodsFieldMapper.insertSelective(record);
    }

    @Override
    public GoodsField queryByPrimaryKey(String guid) {
        return goodsFieldMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsField record) {
        return goodsFieldMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsField record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsField record) {
        return goodsFieldMapper.updateByPrimaryKey(record);
    }

    /**
     * 分页查询产地信息
     *
     * @param goodsFieldForm
     * @return
     */
    @Override
    public PageInfo searchGoodsFieldByPage(GoodsFieldForm goodsFieldForm) {
        PageBean pageBean = new PageBean(goodsFieldForm);
        PageHelper.startPage(goodsFieldForm.getPageNo(),goodsFieldForm.getPageSize());
        Page page =goodsFieldMapper.searchByPage(pageBean);
        List<GoodsFieldForm> list = BeanCopyUtil.convertList(page,GoodsFieldForm.class);
        return new PageInfo(list);
    }

    /**
     * 添加商品产地
     *
     * @param goodsFieldForm
     * @return
     */
    @Override
    public int addGoodsFieldInfo(GoodsFieldForm goodsFieldForm) {
        //TODO 添加产地信息
        GoodsField record = new GoodsField();
        BeanCopyUtil.copyBean(goodsFieldForm,record);
        // 验证唯一
        int i = goodsFieldMapper.isExist(record);
    	if(i > 0) {
    		throw new GlobalException(ResultEnum.FAILED.getCode(), "产地已存在!");
    	}
        i = this.saveSelective(record);
        //TODO 添加产品辅助信息
        List<GoodsInfoAuxiliaryForm> auxList =goodsFieldForm.getAuxList();
        if(null!=auxList && auxList.size()>0){
            for (GoodsInfoAuxiliaryForm bean: auxList) {
            	bean.setFkGoodsGuid(record.getGuid());
                bean.setFkAuxItmeId(goodsFieldForm.getGuid());
                bean.setFkOrgGuid(goodsFieldForm.getFkOrgGuid());
                bean.setState(goodsFieldForm.getState());
                bean.setCreateUser(goodsFieldForm.getCreateUser());
                bean.setCreateTime(goodsFieldForm.getCreateTime());
                bean.setModifiedUser(goodsFieldForm.getModifiedUser());
                bean.setModifiedTime(goodsFieldForm.getModifiedTime());
            }
            goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
        }

        return i;
    }

    /**
     * 修改商品产地信息
     *
     * @param goodsFieldForm
     * @return
     */
    @Override
    public int updateGoodsFieldInfo(GoodsFieldForm goodsFieldForm) {
        //TODO 添加产地信息
        GoodsField record = new GoodsField();
        BeanCopyUtil.copyBean(goodsFieldForm,record);
        int i = goodsFieldMapper.updateByPrimaryKeySelective(record);
        //TODO 删除辅助信息
        List<GoodsInfoAuxiliaryForm> auxList =goodsFieldForm.getAuxList();
        // 如果前端没有传auxList
        if (auxList == null || auxList.size() == 0) {
            goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(goodsFieldForm.getGuid());
        }
        if(null!=auxList && auxList.size()>0){
            goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(goodsFieldForm.getGuid());
            for (GoodsInfoAuxiliaryForm bean: auxList) {
            	bean.setFkGoodsGuid(record.getGuid());
                bean.setFkAuxItmeId(goodsFieldForm.getGuid());
                bean.setFkOrgGuid(goodsFieldForm.getFkOrgGuid());
                bean.setState(1);
                bean.setCreateUser(goodsFieldForm.getModifiedUser());
                bean.setCreateTime(goodsFieldForm.getModifiedTime());
                bean.setModifiedUser(goodsFieldForm.getModifiedUser());
                bean.setModifiedTime(goodsFieldForm.getModifiedTime());
            }
            goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
        }
        return i;
    }

    /**
     * 查询产地明细
     * @param goodsId
     * @param orgGuid
     * @return
     */
    @Override
    public GoodsFieldForm getGoodsFieldInfoById(String goodsId, String orgGuid) {
        GoodsFieldForm goodsFieldForm =  new GoodsFieldForm();
        if(null!=goodsId && !"".equals(goodsId)){
            GoodsField record = goodsFieldMapper.getGoodsFieldInfoById(goodsId,orgGuid);
            if(record!=null){
                //TODO 处理图片信息
                String imgStr = record.getFieldImages();
                List<String> list = ConvertStringToList.getListFormSplitString(imgStr,",");
                BeanCopyUtil.copyBean(record,goodsFieldForm);
                //TODO 补充产地信息
                List<GoodsInfoAuxiliary> auxList =  goodsInfoAuxiliaryMapper.queryListByParentId(record.getGuid());
                goodsFieldForm.setImagesList(list);
                if(null!= auxList && auxList.size()>0){
                   List<GoodsInfoAuxiliaryForm> auxFormList =  BeanCopyUtil.convertList(auxList,GoodsInfoAuxiliaryForm.class);
                   goodsFieldForm.setAuxList(auxFormList);
                }
                //TODO 处理地址信息
                dealPositionInfo(goodsFieldForm);
                return goodsFieldForm;
            }
        }
       return null;
    }

    private void dealPositionInfo(GoodsFieldForm goodsFieldForm){
        String positionId = goodsFieldForm.getFkPostionId();
        if(null!= positionId && !"".equals(positionId)){
            int nextLen =0;
            int totalLen = positionId.length();
            String provinceId = positionId.substring(0,POSITION_HEAD_LENGTH);
            goodsFieldForm.setProvinceId(provinceId);
            int currentLen = totalLen-POSITION_HEAD_LENGTH;
            if(currentLen>=POSITION_LEVEL_STEP){
                nextLen = POSITION_HEAD_LENGTH+POSITION_LEVEL_STEP;
                String cityId = positionId.substring(0,nextLen);
                goodsFieldForm.setCityId(cityId);
                currentLen = totalLen-cityId.length();
                if(currentLen>=POSITION_LEVEL_STEP){
                    nextLen = cityId.length()+POSITION_LEVEL_STEP;
                    String countyId = positionId.substring(0,nextLen);
                    goodsFieldForm.setCountyId(countyId);
                }
            }
        }

    }
}
