package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.TracedCodeUtils;
import com.hcloud.microservice.goods.bo.GoodsQRCodeInfoBO;
import com.hcloud.microservice.goods.dao.GoodsQRCodeInfoMapper;
import com.hcloud.microservice.goods.dao.GoodsQRCodeRecordMapper;
import com.hcloud.microservice.goods.enums.TracedCodeImgInfoEnum;
import com.hcloud.microservice.goods.event.TracedQRCodeImgEvent;
import com.hcloud.microservice.goods.facade.entity.GoodsDealer;
import com.hcloud.microservice.goods.facade.entity.GoodsProInputs;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeInfo;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeRecord;
import com.hcloud.microservice.goods.facade.forms.*;
import com.hcloud.microservice.goods.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @author apple
 */
@Slf4j
@Transactional(rollbackFor = Exception.class)
@Service("goodsQRCodeInfoService")
public class GoodsQRCodeInfoServiceImpl implements GoodsQRCodeInfoService {

    /**
     * 二维码流水号长度
     */
    private static final Integer SN_LENGTH=5;

    private static final Integer PROCESS_SOURCE_TYPE=1;

    private static final Integer PROCESS_PRODUCT_TYPE=2;

    @Autowired
    private ApplicationContext applicationContext;

    @Resource
    private GoodsQRCodeInfoMapper goodsQRCodeInfoMapper;

    @Resource
    private GoodsQRCodeRecordMapper goodsQRCodeRecordMapper;

    @Autowired
    private GoodsBatchInfoService goodsBatchInfoService;

    @Autowired
    private GoodsFieldService goodsFieldService;

    @Autowired
    private GoodsAssayInfoService goodsAssayInfoService;

    @Autowired
    private GoodsProInputsService goodsProInputsService;

    @Autowired
    private GoodsProNormalService goodsProNormalService;

    @Autowired
    private RedisUtils redisUtils;
    
    //@Autowired
    //private GoodsDealerService goodsDealerService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updateGoodsQRInfoByScan(String qrToken) {
        if(null!=qrToken && !"".equals(qrToken)){
            //查询溯源码明细信息
            GoodsQRCodeInfoBO codeInfo = goodsQRCodeInfoMapper.getGoodsQRInfoByQrToken(qrToken);
            if(codeInfo!=null){
                //查询溯源码生成记录
                GoodsQRCodeRecord goodsQRCodeRecord =goodsQRCodeRecordMapper.selectByPrimaryKey(codeInfo.getFkBatchRecordGuid());
                if(goodsQRCodeRecord==null){throw new RuntimeException();}
                //判断是否是首次扫码
                if(codeInfo.getScanFirstTimed()==null){
                    int usedCount =goodsQRCodeRecord.getUsedNum();
                    usedCount++;
                    codeInfo.setScanFirstTimed(new Date());
                    goodsQRCodeRecord.setUsedNum(usedCount);
                }
                //累加扫码次数，更新最后扫码时间
                Integer totalScount =goodsQRCodeRecord.getScanNum();
                Integer scount = codeInfo.getScanCount();
                totalScount++;
                scount++;
                goodsQRCodeRecord.setScanNum(totalScount);
                codeInfo.setScanCount(scount);
                codeInfo.setScanLastTimed(new Date());
                //更新溯源码信息
                GoodsQRCodeInfo updateInfo = new GoodsQRCodeInfo();
                BeanCopyUtil.copyBean(codeInfo,updateInfo);
                int i = goodsQRCodeInfoMapper.updateByPrimaryKeySelective(updateInfo);
                if(i==0){
                    throw new RuntimeException();
                }
                //更新溯源码生成记录
                return goodsQRCodeRecordMapper.updateByPrimaryKeySelective(goodsQRCodeRecord);
            }
        }
        return 0;
    }

    @Override
    public GoodsQRCodeInfoForm getGoodsQRInfo(String qrToken) {
        GoodsQRCodeInfoForm cacheObj = (GoodsQRCodeInfoForm) redisUtils.getValue(qrToken);
        GoodsQRCodeInfoBO codeInfo =  goodsQRCodeInfoMapper.getGoodsQRInfoByQrToken(qrToken);
        if(cacheObj!=null){
        	BeanCopyUtil.copyNotNullProperties(codeInfo, cacheObj);
        	return cacheObj;
        }
        GoodsQRCodeInfoForm resultForm = new GoodsQRCodeInfoForm();
        if(null!=codeInfo){
            BeanCopyUtil.copyBean(codeInfo,resultForm);
            String recordId = resultForm.getFkBatchRecordGuid();
            GoodsQRCodeRecord goodsQRCodeRecord = goodsQRCodeRecordMapper.selectByPrimaryKey(recordId);
            String goodsBatchId =  goodsQRCodeRecord.getFkGoodsBatchGuid();
            GoodsBatchInfoForm goodsBatchInfoForm = goodsBatchInfoService.searchGoodsBatchInfoById(goodsBatchId);
            String fieldId = goodsBatchInfoForm.getFkFieldGuid();
            String batchId = goodsBatchInfoForm.getGuid();
            GoodsFieldForm goodsFieldForm = goodsFieldService.getGoodsFieldInfoById(fieldId,codeInfo.getFkOrgGuid());
            GoodsAssayInfoForm assayInfoForm = goodsAssayInfoService.searchByGoodsBatchId(batchId);
            List<GoodsProInputsForm> inputsList = goodsProInputsService.searchProInputsByBatchGuid(batchId);
            List<GoodsProNormalForm> sourceList = goodsProNormalService.searchProNormalByFK(batchId,PROCESS_SOURCE_TYPE);
            List<GoodsProNormalForm> productList = goodsProNormalService.searchProNormalByFK(batchId,PROCESS_PRODUCT_TYPE);
            //List<GoodsDealer> dealerList = goodsDealerService.selectList(codeInfo.getFkOrgGuid());
            resultForm.setGoodsBatchInfoForm(goodsBatchInfoForm);
            resultForm.setGoodsFieldForm(goodsFieldForm);
            resultForm.setAssayInfoForm(assayInfoForm);
            resultForm.setInputsList(inputsList);
            resultForm.setSourceList(sourceList);
            resultForm.setProductList(productList);
            //resultForm.setDealerList(dealerList);
            redisUtils.setValue(qrToken,resultForm,600,TimeUnit.SECONDS);
        }
        return resultForm;
    }

    /**
     * 批量保存溯源码信息
     *
     * @param codeInfoForm
     * @return
     */
    @Override
    public int batchSaveGoodsQRCodeInfo(GoodsQRCodeInfoForm codeInfoForm) {
        if(codeInfoForm!=null && codeInfoForm.getCodeCount()>0){
            List<GoodsQRCodeInfo> insertList = Lists.newArrayList();
            List<Map> qrcodeImgList = Lists.newArrayList();
            //查询商品批次信息
            GoodsBatchInfoForm batchInfoForm = goodsBatchInfoService.searchGoodsBatchInfoById(codeInfoForm.getGoodsBatchInfoId());
            if(null!=batchInfoForm){
                //查询生成二维码记录信息
                List<String> containerList = Lists.newArrayList();
                Integer codeCount = codeInfoForm.getCodeCount();
                String goodsId = batchInfoForm.getFkGoodsGuid();
                for(int i=0;i<codeCount;i++){
                    Map map = new ConcurrentHashMap(4);
                    GoodsQRCodeInfo info = new GoodsQRCodeInfo();
                    info.setFkBatchRecordGuid(codeInfoForm.getFkBatchRecordGuid());
                    info.setFkOrgGuid(codeInfoForm.getFkOrgGuid());
                    info.setFkGoodsGuid(goodsId);
                    info.setScanCount(0);
                    info.setCreateTime(new Date());
                    info.setCreateUser(codeInfoForm.getCreateUser());
                    info.setModifiedTime(new Date());
                    info.setModifiedUser(codeInfoForm.getModifiedUser());
                    info.setState(1);
                    //二维码信息
                    getQrcode(codeInfoForm,info,String.valueOf(i+1),containerList);
                    if(insertList.size()>2000){
                        goodsQRCodeInfoMapper.batchSaveQRCodeInfo(insertList);
                        insertList.clear();
                    }
                    insertList.add(info);
                    //组装二维码生成图片信息
                    map.put(TracedCodeImgInfoEnum.IMG_SUB_DIR_NAME,codeInfoForm.getQrcodeParentCode());
                    map.put(TracedCodeImgInfoEnum.CODE_CONTENT,info.getQrcodeUrl());
                    map.put(TracedCodeImgInfoEnum.ORG_LOGO_NAME,codeInfoForm.getIco()==null?"logo.png":codeInfoForm.getIco());
                    map.put(TracedCodeImgInfoEnum.IMG_NAME,info.getQrcodeImg());
                    qrcodeImgList.add(map);
                }
                //生成二维码记录
                goodsQRCodeInfoMapper.batchSaveQRCodeInfo(insertList);
                //异步生成二维码图片
                applicationContext.publishEvent(new TracedQRCodeImgEvent(new Object(),qrcodeImgList));
                return 1;
            }
        }
        return 0;
    }

    /**
     * 通过溯源码生成记录Id查询溯源码信息
     *
     * @param goodsQRCodeInfoForm
     * @return
     */
    @Override
    public PageInfo queryGoodsQRCodeInfoByRecId(GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
        PageHelper.startPage(goodsQRCodeInfoForm.getPageNum(),goodsQRCodeInfoForm.getPageSize());
        List<GoodsQRCodeInfoBO> list = goodsQRCodeInfoMapper.queryGoodsQRCodeInfoByRecId(goodsQRCodeInfoForm.getFkBatchRecordGuid());
        return new PageInfo(list);
    }

    /**
     * 删除溯源明细信息（逻辑删除）
     *
     * @param goodsQRCodeInfoForm
     * @return
     */
    @Override
    public int delGoodsQRCodeInfoByRecordId(GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
        GoodsQRCodeInfo codeInfo = new GoodsQRCodeInfo();
        BeanCopyUtil.copyBean(goodsQRCodeInfoForm,codeInfo);
        return  goodsQRCodeInfoMapper.updateGoodsQRCodeInfoByRecord(codeInfo);
    }

    /**
     * 删除溯源明细信息（物理删除）
     *
     * @param recordId
     * @return
     */
    @Override
    public int delQRCodeInfoByPhysical(String recordId) {
        return goodsQRCodeInfoMapper.delQRCodeInfoByRecordId(recordId);
    }

    /**
     * 废弃单个溯源码信息（逻辑删除）
     * @param goodsQRCodeInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int forbiddenCodeInfoByCodeId(GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
        //废弃单个溯源码信息,更新状态
       String codeInfoId = goodsQRCodeInfoForm.getGuid();
       GoodsQRCodeInfo goodsQRCodeInfo =  goodsQRCodeInfoMapper.selectByPrimaryKey(codeInfoId);
       if(goodsQRCodeInfo!=null){
           String batchRecord = goodsQRCodeInfo.getFkBatchRecordGuid();
           goodsQRCodeInfo.setState(0);
           goodsQRCodeInfo.setModifiedUser(goodsQRCodeInfoForm.getModifiedUser());
           goodsQRCodeInfo.setModifiedTime(new Date());
           int i = goodsQRCodeInfoMapper.updateByPrimaryKeySelective(goodsQRCodeInfo);
           if(i>0){
               //更新废弃数量
               GoodsQRCodeRecord record = goodsQRCodeRecordMapper.selectByPrimaryKey(batchRecord);
               if(record!=null){
                   int count = record.getCancelNum();
                   count++;
                   record.setCancelNum(count);
                   record.setModifiedTime(new Date());
                   record.setModifiedUser(goodsQRCodeInfoForm.getModifiedUser());
                   goodsQRCodeRecordMapper.updateByPrimaryKeySelective(record);
               }
           }
           return i;
       }

        return 0;
    }

    /**
     * 批量更新溯源码信息通过生成记录Id
     *
     * @param list
     * @return
     */
    @Override
    public int batchUpdateGoodsQRCodeInfoByRecord(List<String> list) {
        return goodsQRCodeInfoMapper.batchUpdateGoodsQRCodeInfoByRecord(list);
    }

    /**
     * 获取溯源码
     * @param codeInfoForm
     * @param info
     * @param currentNum
     */
    private void getQrcode(GoodsQRCodeInfoForm codeInfoForm,GoodsQRCodeInfo info,String currentNum, List<String> containerList){
        if(null!= codeInfoForm && !"".equals(codeInfoForm.getQrcodeParentCode())){
            StringBuilder stb = new StringBuilder();
            String parentCode = codeInfoForm.getQrcodeParentCode();
            int len = SN_LENGTH-currentNum.length();
            String sn="";
            for (int i = 0; i < len; i++) {
                sn+="0";
            }
            String qrcode = parentCode+sn+currentNum;
            String codeSuffix = sn+currentNum;
            info.setQrcode(qrcode);
            info.setQrcodeToken(TracedCodeUtils.generalTracedCode(codeInfoForm.getOrgBrevity(),codeSuffix,containerList));
            info.setQrcodeImg(info.getQrcodeToken()+".png");
            stb.append(codeInfoForm.getCodeUrlPrefix()).append("?qrToken=").append(info.getQrcodeToken());
            info.setQrcodeUrl(stb.toString());
        }
    }








    @Override
    public int delByPrimaryKey(String guid) {
        return goodsQRCodeInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsQRCodeInfo record) {
        return goodsQRCodeInfoMapper.insert(record);
    }

    @Override
    public int saveSelective(GoodsQRCodeInfo record) {
        return goodsQRCodeInfoMapper.insertSelective(record);
    }

    @Override
    public GoodsQRCodeInfo queryByPrimaryKey(String guid) {
        return goodsQRCodeInfoMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsQRCodeInfo record) {
        return goodsQRCodeInfoMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsQRCodeInfo record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsQRCodeInfo record) {
        return goodsQRCodeInfoMapper.updateByPrimaryKey(record);
    }
    
    @Override
    public PageInfo getPrintGoodsQRInfo(GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
    	PageHelper.startPage(goodsQRCodeInfoForm.getPageNum(),goodsQRCodeInfoForm.getPageSize());
        List<GoodsQRCodeInfoBO> list =  goodsQRCodeInfoMapper.getPrintGoodsQRInfo(goodsQRCodeInfoForm);
    	return new PageInfo<>(list);
    }

	@Override
	@Transactional
	public int updateQRIsPrint(List<String> ids) {
		// TODO Auto-generated method stub
		return goodsQRCodeInfoMapper.updateQRIsPrint(ids);
	}
}
